@@ -15,10 +15,10 @@ static const char *stateToString(int state)
     case ModuleBody: return "ModuleBody";
     case ModuleBodyContains: return "ModuleBodyContains";
     case AttributeList: return "AttributeList";
-    case Variable: return "Variable";
+    case FVariable: return "FVariable";
     case Initialization: return "Initialization";
     case ArrayInitializer: return "ArrayInitializer";
-    case Enum: return "Enum";
+    case FEnum: return "FEnum";
     case Typedef: return "Typedef";
     case TypedefBody: return "TypedefBody";
     case TypedefBodyContains: return "TypedefBodyContains";
