@@ -67,7 +67,7 @@
 
 
 /* First part of user prologue.  */
-#line 1 "mscgen_language.y"
+#line 1 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 
 /***************************************************************************
  *
@@ -294,7 +294,7 @@ Msc MscParse(FILE *in)
 
 
 
-#line 298 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 298 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -1440,208 +1440,208 @@ yyreduce:
   switch (yyn)
     {
   case 2: /* msc: TOK_MSC TOK_OCBRACKET optlist TOK_SEMICOLON entitylist TOK_SEMICOLON arclist TOK_SEMICOLON TOK_CCBRACKET  */
-#line 291 "mscgen_language.y"
+#line 291 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.msc) = MscAlloc((yyvsp[-6].opt), (yyvsp[-4].entitylist), (yyvsp[-2].arclist));
     *(Msc *)yyparse_result = (yyval.msc);
 
 }
-#line 1450 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1450 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 3: /* msc: TOK_MSC TOK_OCBRACKET entitylist TOK_SEMICOLON arclist TOK_SEMICOLON TOK_CCBRACKET  */
-#line 297 "mscgen_language.y"
+#line 297 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.msc) = MscAlloc(NULL, (yyvsp[-4].entitylist), (yyvsp[-2].arclist));
     *(Msc *)yyparse_result = (yyval.msc);
 
 }
-#line 1460 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1460 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 5: /* optlist: optlist TOK_COMMA opt  */
-#line 305 "mscgen_language.y"
+#line 305 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.opt) = MscLinkOpt((yyvsp[-2].opt), (yyvsp[0].opt));
 }
-#line 1468 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1468 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 6: /* opt: optval TOK_EQUAL string  */
-#line 310 "mscgen_language.y"
+#line 310 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.opt) = MscAllocOpt((yyvsp[-2].optType), (yyvsp[0].string));
 }
-#line 1476 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1476 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 11: /* entitylist: entity  */
-#line 317 "mscgen_language.y"
+#line 317 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.entitylist) = MscLinkEntity(NULL, (yyvsp[0].entity));   /* Create new list */
 }
-#line 1484 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1484 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 12: /* entitylist: entitylist TOK_COMMA entity  */
-#line 321 "mscgen_language.y"
+#line 321 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.entitylist) = MscLinkEntity((yyvsp[-2].entitylist), (yyvsp[0].entity));     /* Add to existing list */
 }
-#line 1492 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1492 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 13: /* entity: string  */
-#line 328 "mscgen_language.y"
+#line 328 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.entity) = MscAllocEntity((yyvsp[0].string));
 }
-#line 1500 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1500 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 14: /* entity: entity TOK_OSBRACKET attrlist TOK_CSBRACKET  */
-#line 332 "mscgen_language.y"
+#line 332 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     MscEntityLinkAttrib((yyvsp[-3].entity), (yyvsp[-1].attrib));
 }
-#line 1508 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1508 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 15: /* arclist: arc  */
-#line 338 "mscgen_language.y"
+#line 338 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.arclist) = MscLinkArc(NULL, (yyvsp[0].arc));      /* Create new list */
 }
-#line 1516 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1516 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 16: /* arclist: arclist TOK_SEMICOLON arc  */
-#line 342 "mscgen_language.y"
+#line 342 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.arclist) = MscLinkArc((yyvsp[-2].arclist), (yyvsp[0].arc));     /* Add to existing list */
 }
-#line 1524 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1524 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 17: /* arclist: arclist TOK_COMMA arc  */
-#line 346 "mscgen_language.y"
+#line 346 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     /* Add a special 'parallel' arc */
     (yyval.arclist) = MscLinkArc(MscLinkArc((yyvsp[-2].arclist), MscAllocArc(NULL, NULL, MSC_ARC_PARALLEL, lex_getlinenum())), (yyvsp[0].arc));
 }
-#line 1533 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1533 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 18: /* arc: arcrel TOK_OSBRACKET attrlist TOK_CSBRACKET  */
-#line 355 "mscgen_language.y"
+#line 355 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     MscArcLinkAttrib((yyvsp[-3].arc), (yyvsp[-1].attrib));
 }
-#line 1541 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1541 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 20: /* arcrel: TOK_SPECIAL_ARC  */
-#line 361 "mscgen_language.y"
+#line 361 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.arc) = MscAllocArc(NULL, NULL, (yyvsp[0].arctype), lex_getlinenum());
 }
-#line 1549 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1549 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 21: /* arcrel: string relation_box string  */
-#line 365 "mscgen_language.y"
+#line 365 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.arc) = MscAllocArc((yyvsp[-2].string), (yyvsp[0].string), (yyvsp[-1].arctype), lex_getlinenum());
 }
-#line 1557 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1557 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 22: /* arcrel: string relation_bi string  */
-#line 369 "mscgen_language.y"
+#line 369 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     MscArc arc = MscAllocArc((yyvsp[-2].string), (yyvsp[0].string), (yyvsp[-1].arctype), lex_getlinenum());
     MscArcLinkAttrib(arc, MscAllocAttrib(MSC_ATTR_BI_ARROWS, strdup_s("true")));
     (yyval.arc) = arc;
 }
-#line 1567 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1567 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 23: /* arcrel: string relation_to string  */
-#line 375 "mscgen_language.y"
+#line 375 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.arc) = MscAllocArc((yyvsp[-2].string), (yyvsp[0].string), (yyvsp[-1].arctype), lex_getlinenum());
 }
-#line 1575 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1575 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 24: /* arcrel: string relation_line string  */
-#line 379 "mscgen_language.y"
+#line 379 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     MscArc arc = MscAllocArc((yyvsp[-2].string), (yyvsp[0].string), (yyvsp[-1].arctype), lex_getlinenum());
     MscArcLinkAttrib(arc, MscAllocAttrib(MSC_ATTR_NO_ARROWS, strdup_s("true")));
     (yyval.arc) = arc;
 }
-#line 1585 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1585 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 25: /* arcrel: string relation_from string  */
-#line 385 "mscgen_language.y"
+#line 385 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.arc) = MscAllocArc((yyvsp[0].string), (yyvsp[-2].string), (yyvsp[-1].arctype), lex_getlinenum());
 }
-#line 1593 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1593 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 26: /* arcrel: string relation_to TOK_ASTERISK  */
-#line 389 "mscgen_language.y"
+#line 389 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.arc) = MscAllocArc((yyvsp[-2].string), strdup_s("*"), (yyvsp[-1].arctype), lex_getlinenum());
 }
-#line 1601 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1601 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 27: /* arcrel: TOK_ASTERISK relation_from string  */
-#line 393 "mscgen_language.y"
+#line 393 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.arc) = MscAllocArc((yyvsp[0].string), strdup_s("*"), (yyvsp[-1].arctype), lex_getlinenum());
 }
-#line 1609 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1609 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 54: /* attrlist: attrlist TOK_COMMA attr  */
-#line 405 "mscgen_language.y"
+#line 405 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.attrib) = MscLinkAttrib((yyvsp[-2].attrib), (yyvsp[0].attrib));
 }
-#line 1617 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1617 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 55: /* attr: attrval TOK_EQUAL string  */
-#line 410 "mscgen_language.y"
+#line 410 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.attrib) = MscAllocAttrib((yyvsp[-2].attribType), (yyvsp[0].string));
 }
-#line 1625 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1625 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 67: /* string: TOK_QSTRING  */
-#line 421 "mscgen_language.y"
+#line 421 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.string) = removeEscapes((yyvsp[0].string));
 }
-#line 1633 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1633 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
   case 68: /* string: TOK_STRING  */
-#line 425 "mscgen_language.y"
+#line 425 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 {
     (yyval.string) = (yyvsp[0].string);
 }
-#line 1641 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1641 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
     break;
 
 
-#line 1645 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.9.7-1.x86_64/build/generated_src/mscgen_language.cpp"
+#line 1645 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/build/generated_src/mscgen_language.cpp"
 
       default: break;
     }
@@ -1834,7 +1834,7 @@ yyreturnlab:
   return yyresult;
 }
 
-#line 428 "mscgen_language.y"
+#line 428 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.10.0-1.x86_64/src/doxygen-1.10.0/deps/libmscgen/mscgen_language.y"
 
 
 
