@@ -125,17 +125,11 @@ void ClangTUParser::parse()
   p->index    = clang_createIndex(0, 0);
   p->curToken = 0;
   p->cursors.clear();
-  size_t clang_option_len = 0;
   std::vector<clang::tooling::CompileCommand> command;
   if (p->parser.database()!=nullptr)
   {
     // check if the file we are parsing is in the DB
     command = p->parser.database()->getCompileCommands(fileName.data());
-    if (!command.empty() )
-    {
-      // it's possible to have multiple entries for the same file, so use the last entry
-      clang_option_len = command[command.size()-1].CommandLine.size();
-    }
   }
   std::vector<char *> argv;
   if (!command.empty() )
@@ -506,7 +500,7 @@ void ClangTUParser::codifyLines(OutputCodeList &ol,const FileDef *fd,const char
       free(tmp);
       if (fontClass) ol.endFontClass();
       ol.endCodeLine();
-      ol.startCodeLine(TRUE);
+      ol.startCodeLine(line);
       writeLineNumber(ol,fd,line,inlineCodeFragment);
       if (fontClass) ol.startFontClass(fontClass);
     }
@@ -548,7 +542,7 @@ void ClangTUParser::writeMultiLineCodeLink(OutputCodeList &ol,
       //printf("writeCodeLink(%s,%s,%s,%s)\n",ref,file,anchor,sp);
       ol.writeCodeLink(d->codeSymbolType(),ref,file,anchor,QCString(sp,p-sp-1),tooltip);
       ol.endCodeLine();
-      ol.startCodeLine(TRUE);
+      ol.startCodeLine(line);
       writeLineNumber(ol,fd,line,inlineCodeFragment);
     }
     else
@@ -712,7 +706,7 @@ void ClangTUParser::writeSources(OutputCodeList &ol,const FileDef *fd)
   unsigned int line=1,column=1;
   QCString lineNumber,lineAnchor;
   bool inlineCodeFragment = false;
-  ol.startCodeLine(TRUE);
+  ol.startCodeLine(line);
   writeLineNumber(ol,fd,line,!inlineCodeFragment);
   for (unsigned int i=0;i<p->numTokens;i++)
   {
@@ -724,7 +718,7 @@ void ClangTUParser::writeSources(OutputCodeList &ol,const FileDef *fd)
     {
       line++;
       ol.endCodeLine();
-      ol.startCodeLine(TRUE);
+      ol.startCodeLine(line);
       writeLineNumber(ol,fd,line,!inlineCodeFragment);
     }
     while (column<c) { ol.codify(" "); column++; }
