@@ -35,37 +35,39 @@ class ConceptDefImpl : public DefinitionMixin<ConceptDefMutable>
                    const QCString &name,const QCString &tagRef=QCString(),const QCString &tagFile=QCString());
    ~ConceptDefImpl();
 
-    virtual ConceptDef *resolveAlias() { return this; }
+    ConceptDef *resolveAlias() { return this; }
 
     //---------- ConceptDef
-    virtual DefType definitionType() const;
-    virtual CodeSymbolType codeSymbolType() const { return CodeSymbolType::Concept; }
-    virtual QCString getOutputFileBase() const;
-    virtual bool hasDetailedDescription() const;
-    virtual QCString displayName(bool includeScope=true) const;
-    virtual const IncludeInfo *includeInfo() const;
-    virtual ArgumentList getTemplateParameterList() const;
-    virtual QCString anchor() const;
-    virtual bool isLinkableInProject() const;
-    virtual bool isLinkable() const;
-    virtual QCString initializer() const;
-    virtual void writeDeclarationLink(OutputList &ol,bool &found,
-                              const QCString &header,bool localNames) const;
-    virtual const NamespaceDef *getNamespaceDef() const;
-    virtual const FileDef *getFileDef() const;
-    virtual QCString title() const;
-    virtual int groupId() const;
+    DefType definitionType() const override;
+    CodeSymbolType codeSymbolType() const override { return CodeSymbolType::Concept; }
+    QCString getOutputFileBase() const override;
+    bool hasDetailedDescription() const override;
+    QCString displayName(bool includeScope=true) const override;
+    const IncludeInfo *includeInfo() const override;
+    ArgumentList getTemplateParameterList() const override;
+    QCString anchor() const override;
+    bool isLinkableInProject() const override;
+    bool isLinkable() const override;
+    QCString initializer() const override;
+    void writeDeclarationLink(OutputList &ol,bool &found,
+                              const QCString &header,bool localNames) const override;
+    const NamespaceDef *getNamespaceDef() const override;
+    const FileDef *getFileDef() const override;
+    const ModuleDef *getModuleDef() const override;
+    QCString title() const override;
+    int groupId() const override;
 
     //---------- ConceptDefMutable
-    virtual void setIncludeFile(FileDef *fd,const QCString &incName,bool local,bool force);
-    virtual void setTemplateArguments(const ArgumentList &al);
-    virtual void setNamespace(NamespaceDef *nd);
-    virtual void setFileDef(FileDef *fd);
-    virtual void writeTagFile(TextStream &);
-    virtual void writeDocumentation(OutputList &);
-    virtual void setInitializer(const QCString &init);
-    virtual void findSectionsInDocumentation();
-    virtual void setGroupId(int id);
+    void setIncludeFile(FileDef *fd,const QCString &incName,bool local,bool force) override;
+    void setTemplateArguments(const ArgumentList &al) override;
+    void setNamespace(NamespaceDef *nd) override;
+    void setFileDef(FileDef *fd) override;
+    void setModuleDef(ModuleDef *mod) override;
+    void writeTagFile(TextStream &) override;
+    void writeDocumentation(OutputList &) override;
+    void setInitializer(const QCString &init) override;
+    void findSectionsInDocumentation() override;
+    void setGroupId(int id) override;
 
     //---------- Helpers
     void writeBriefDescription(OutputList &) const;
@@ -73,12 +75,14 @@ class ConceptDefImpl : public DefinitionMixin<ConceptDefMutable>
     void writeIncludeFiles(OutputList &ol) const;
     void writeDetailedDescription(OutputList &ol,const QCString &title) const;
     void writeDefinition(OutputList &ol,const QCString &title) const;
+    void addConceptAttributes(OutputList &ol) const;
 
   private:
     QCString                     m_fileName;
     std::unique_ptr<IncludeInfo> m_incInfo;
     NamespaceDef                *m_nspace = 0;
     FileDef                     *m_fileDef = 0;
+    ModuleDef                   *m_moduleDef = 0;
     ArgumentList                 m_tArgList;
     QCString                     m_initializer;
     int                          m_groupId = -1;
@@ -98,46 +102,48 @@ class ConceptDefAliasImpl : public DefinitionAliasMixin<ConceptDef>
   public:
     ConceptDefAliasImpl(const Definition *newScope,const ConceptDef *cd)
       : DefinitionAliasMixin(newScope,cd) { init(); }
-    virtual ~ConceptDefAliasImpl() { deinit(); }
+    ~ConceptDefAliasImpl() override { deinit(); }
 
     const ConceptDef *getCdAlias() const { return toConceptDef(getAlias()); }
-    virtual ConceptDef *resolveAlias() { return const_cast<ConceptDef*>(getCdAlias()); }
+    ConceptDef *resolveAlias() { return const_cast<ConceptDef*>(getCdAlias()); }
 
-    virtual DefType definitionType() const { return TypeConcept; }
-    virtual CodeSymbolType codeSymbolType() const
+    DefType definitionType() const override { return TypeConcept; }
+    CodeSymbolType codeSymbolType() const override
     { return getCdAlias()->codeSymbolType(); }
-    virtual QCString getOutputFileBase() const
+    QCString getOutputFileBase() const override
     { return getCdAlias()->getOutputFileBase(); }
-    virtual QCString getReference() const
+    QCString getReference() const override
     { return getCdAlias()->getReference(); }
-    virtual bool isReference() const
+    bool isReference() const override
     { return getCdAlias()->isReference(); }
-    virtual bool hasDetailedDescription() const
+    bool hasDetailedDescription() const override
     { return getCdAlias()->hasDetailedDescription(); }
-    virtual QCString displayName(bool includeScope=true) const
+    QCString displayName(bool includeScope=true) const override
     { return getCdAlias()->displayName(includeScope); }
-    virtual const IncludeInfo *includeInfo() const
+    const IncludeInfo *includeInfo() const override
     { return getCdAlias()->includeInfo(); }
-    virtual ArgumentList getTemplateParameterList() const
+    ArgumentList getTemplateParameterList() const override
     { return getCdAlias()->getTemplateParameterList(); }
-    virtual QCString anchor() const
+    QCString anchor() const override
     { return getCdAlias()->anchor(); }
-    virtual bool isLinkableInProject() const
+    bool isLinkableInProject() const override
     { return getCdAlias()->isLinkableInProject(); }
-    virtual bool isLinkable() const
+    bool isLinkable() const override
     { return getCdAlias()->isLinkable(); }
-    virtual QCString initializer() const
+    QCString initializer() const override
     { return getCdAlias()->initializer(); }
-    virtual const NamespaceDef *getNamespaceDef() const
+    const NamespaceDef *getNamespaceDef() const override
     { return getCdAlias()->getNamespaceDef(); }
-    virtual const FileDef *getFileDef() const
+    const FileDef *getFileDef() const override
     { return getCdAlias()->getFileDef(); }
-    virtual QCString title() const
+    const ModuleDef *getModuleDef() const override
+    { return getCdAlias()->getModuleDef(); }
+    QCString title() const override
     { return getCdAlias()->title(); }
-    virtual void writeDeclarationLink(OutputList &ol,bool &found,
-                              const QCString &header,bool localNames) const
+    void writeDeclarationLink(OutputList &ol,bool &found,
+                              const QCString &header,bool localNames) const override
     { getCdAlias()->writeDeclarationLink(ol,found,header,localNames); }
-    virtual int groupId() const
+    int groupId() const override
     { return getCdAlias()->groupId(); }
 };
 
@@ -234,12 +240,12 @@ void ConceptDefImpl::setIncludeFile(FileDef *fd,const QCString &incName,bool loc
   {
     m_incInfo->fileDef     = fd;
     m_incInfo->includeName = incName;
-    m_incInfo->local       = local;
+    m_incInfo->kind        = local ? IncludeKind::IncludeLocal : IncludeKind::IncludeSystem;
   }
   if (force && !incName.isEmpty())
   {
     m_incInfo->includeName = incName;
-    m_incInfo->local       = local;
+    m_incInfo->kind        = local ? IncludeKind::IncludeLocal : IncludeKind::IncludeSystem;
   }
 }
 
@@ -263,11 +269,21 @@ void ConceptDefImpl::setFileDef(FileDef *fd)
   m_fileDef=fd;
 }
 
+void ConceptDefImpl::setModuleDef(ModuleDef *mod)
+{
+  m_moduleDef=mod;
+}
+
 const FileDef *ConceptDefImpl::getFileDef() const
 {
   return m_fileDef;
 }
 
+const ModuleDef *ConceptDefImpl::getModuleDef() const
+{
+  return m_moduleDef;
+}
+
 QCString ConceptDefImpl::title() const
 {
   return theTranslator->trConceptReference(displayName());
@@ -348,11 +364,8 @@ void ConceptDefImpl::writeIncludeFiles(OutputList &ol) const
     {
       ol.startParagraph();
       ol.startTypewriter();
-      ol.docify("#include ");
-      if (m_incInfo->local)
-        ol.docify("\"");
-      else
-        ol.docify("<");
+      ol.docify(::includeStatement(SrcLangExt_Cpp,m_incInfo->kind));
+      ol.docify(::includeOpen(SrcLangExt_Cpp,m_incInfo->kind));
       ol.pushGeneratorState();
       ol.disable(OutputType::Html);
       ol.docify(nm);
@@ -367,10 +380,7 @@ void ConceptDefImpl::writeIncludeFiles(OutputList &ol) const
         ol.docify(nm);
       }
       ol.popGeneratorState();
-      if (m_incInfo->local)
-        ol.docify("\"");
-      else
-        ol.docify(">");
+      ol.docify(::includeClose(SrcLangExt_Cpp,m_incInfo->kind));
       ol.endTypewriter();
       ol.endParagraph();
     }
@@ -488,6 +498,19 @@ void ConceptDefImpl::writeAuthorSection(OutputList &ol) const
   ol.popGeneratorState();
 }
 
+void ConceptDefImpl::addConceptAttributes(OutputList &ol) const
+{
+  if (isExported())
+  {
+    ol.pushGeneratorState();
+    ol.disableAllBut(OutputType::Html);
+    ol.startLabels();
+    ol.writeLabel("export",false);
+    ol.endLabels();
+    ol.popGeneratorState();
+  }
+}
+
 void ConceptDefImpl::writeDocumentation(OutputList &ol)
 {
   bool generateTreeView = Config_getBool(GENERATE_TREEVIEW);
@@ -507,6 +530,7 @@ void ConceptDefImpl::writeDocumentation(OutputList &ol)
   // ---- title part
   startTitle(ol,getOutputFileBase(),this);
   ol.parseText(pageTitle);
+  addConceptAttributes(ol);
   addGroupListToTitle(ol,this);
   endTitle(ol,getOutputFileBase(),displayName());
 
@@ -571,6 +595,7 @@ void ConceptDefImpl::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::FileSourceLink:
       case LayoutDocEntry::GroupClasses:
       case LayoutDocEntry::GroupConcepts:
+      case LayoutDocEntry::GroupModules:
       case LayoutDocEntry::GroupInlineClasses:
       case LayoutDocEntry::GroupNamespaces:
       case LayoutDocEntry::GroupDirs:
@@ -578,11 +603,15 @@ void ConceptDefImpl::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::GroupFiles:
       case LayoutDocEntry::GroupGraph:
       case LayoutDocEntry::GroupPageDocs:
+      case LayoutDocEntry::ModuleExports:
+      case LayoutDocEntry::ModuleClasses:
+      case LayoutDocEntry::ModuleConcepts:
+      case LayoutDocEntry::ModuleUsedFiles:
       case LayoutDocEntry::DirSubDirs:
       case LayoutDocEntry::DirFiles:
       case LayoutDocEntry::DirGraph:
         err("Internal inconsistency: member %d should not be part of "
-            "LayoutDocManager::Namespace entry list\n",lde->kind());
+            "LayoutDocManager::Concept entry list\n",lde->kind());
         break;
     }
   }
