@@ -37,7 +37,7 @@ void DocGroup::leaveFile(const QCString &fileName,int line)
 {
   //if (m_memberGroupId!=DOX_NOGROUP)
   //{
-  //  warn(fileName,line,"end of file while inside a member group\n");
+  //  warn(fileName,line,"end of file while inside a member group");
   //}
   m_memberGroupId=DOX_NOGROUP;
   m_memberGroupRelates.resize(0);
@@ -56,7 +56,7 @@ void DocGroup::enterCompound(const QCString &fileName,int line,const QCString &n
 {
   if (m_memberGroupId!=DOX_NOGROUP)
   {
-    warn(fileName,line,"try to put compound %s inside a member group\n",qPrint(name));
+    warn(fileName,line,"try to put compound %s inside a member group",qPrint(name));
   }
   m_memberGroupId=DOX_NOGROUP;
   m_memberGroupRelates.resize(0);
@@ -110,7 +110,7 @@ void DocGroup::open(Entry *e,const QCString &,int, bool implicit)
   if (!implicit) m_openCount++;
   //printf("==> openGroup(name=%s,sec=%x) m_autoGroupStack=%zu\n",
   //  	qPrint(e->name),e->section,m_autoGroupStack.size());
-  if (e->section==Entry::GROUPDOC_SEC) // auto group
+  if (e->section.isGroupDoc()) // auto group
   {
     m_autoGroupStack.push_back(Grouping(e->name,e->groupingPri()));
   }
@@ -200,7 +200,7 @@ void DocGroup::initGroupInfo(Entry *e)
 
 void DocGroup::addDocs(Entry *e)
 {
-  if (e->section==Entry::MEMBERGRP_SEC)
+  if (e->section.isMemberGrp())
   {
     m_memberGroupDocs=e->brief.stripWhiteSpace();
     e->doc = stripLeadingAndTrailingEmptyLines(e->doc,e->docLine);
