@@ -39,6 +39,7 @@
 #include "emoji.h"
 #include "dir.h"
 #include "portable.h"
+#include "moduledef.h"
 
 #define PERLOUTPUT_MAX_INDENTATION 40
 
@@ -46,9 +47,10 @@ class PerlModOutputStream
 {
   public:
     //QCString m_s;
-    std::ostream &m_t;
+    std::ostream *m_t;
 
-    PerlModOutputStream(std::ostream &t) : m_t(t) { }
+    PerlModOutputStream(std::ostream &t) : m_t(&t) { }
+   ~PerlModOutputStream() { m_t=0; }
 
     void add(char c);
     void add(const QCString &s);
@@ -58,38 +60,22 @@ class PerlModOutputStream
 
 void PerlModOutputStream::add(char c)
 {
-  m_t << c;
-  //if (m_t != 0)
-  //  (*m_t) << c;
-  //else
-  //  m_s += c;
+  *m_t << c;
 }
 
 void PerlModOutputStream::add(const QCString &s)
 {
-  m_t << s;
-  //if (m_t != 0)
-  //  (*m_t) << s;
-  //else
-  //  m_s += s;
+  *m_t << s;
 }
 
 void PerlModOutputStream::add(int n)
 {
-  m_t << n;
-  //if (m_t != 0)
-  //  (*m_t) << n;
-  //else
-  //  m_s += QCString().setNum(n);
+  *m_t << n;
 }
 
 void PerlModOutputStream::add(unsigned int n)
 {
-  m_t << n;
-  //if (m_t != 0)
-  //  (*m_t) << n;
-  //else
-  //  m_s += QCString().setNum(n);
+  *m_t << n;
 }
 
 class PerlModOutput
@@ -104,7 +90,9 @@ public:
     m_spaces[0] = 0;
   }
 
-  virtual ~PerlModOutput() { }
+  virtual ~PerlModOutput() { reset(); }
+
+  void reset() { m_stream=0; }
 
   inline void setPerlModOutputStream(PerlModOutputStream *os) { m_stream = os; }
 
@@ -696,12 +684,7 @@ void PerlModDocVisitor::operator()(const DocInclude &inc)
     case DocInclude::VerbInclude:	type = "preformatted"; break;
     case DocInclude::Snippet: return;
     case DocInclude::SnippetTrimLeft: return;
-    case DocInclude::SnipWithLines: return;
-    case DocInclude::SnippetDoc:
-    case DocInclude::IncludeDoc:
-      err("Internal inconsistency: found switch SnippetDoc / IncludeDoc in file: %s"
-          "Please create a bug report\n",__FILE__);
-      break;
+    case DocInclude::SnippetWithLines: return;
   }
   openItem(type);
   m_output.addFieldQuotedString("content", inc.text());
@@ -881,7 +864,11 @@ void PerlModDocVisitor::operator()(const DocSection &s)
 {
   QCString sect = QCString().sprintf("sect%d",s.level());
   openItem(sect);
-  m_output.addFieldQuotedString("title", s.title());
+  //m_output.addFieldQuotedString("title", s.title());
+  if (s.title())
+  {
+    std::visit(*this,*s.title());
+  }
   openSubBlock("content");
   visitChildren(s);
   closeSubBlock();
@@ -1434,6 +1421,7 @@ public:
   void addIncludeInfo(const IncludeInfo *ii);
   void generatePerlModForClass(const ClassDef *cd);
   void generatePerlModForConcept(const ConceptDef *cd);
+  void generatePerlModForModule(const ModuleDef *mod);
   void generatePerlModForNamespace(const NamespaceDef *nd);
   void generatePerlModForFile(const FileDef *fd);
   void generatePerlModForGroup(const GroupDef *gd);
@@ -1709,7 +1697,7 @@ void PerlModGenerator::addIncludeInfo(const IncludeInfo *ii)
     if (!nm.isEmpty())
     {
       m_output.openHash("includes");
-      m_output.addFieldBoolean("local", ii->local)
+      m_output.addFieldBoolean("local", ii->kind==IncludeKind::IncludeLocal || ii->kind==IncludeKind::ImportLocal)
 	.addFieldQuotedString("name", nm)
 	.closeHash();
     }
@@ -1858,6 +1846,66 @@ void PerlModGenerator::generatePerlModForConcept(const ConceptDef *cd)
   m_output.closeHash();
 }
 
+void PerlModGenerator::generatePerlModForModule(const ModuleDef *mod)
+{
+  // + contained class definitions
+  // + contained concept definitions
+  // + member groups
+  // + normal members
+  // + brief desc
+  // + detailed desc
+  // + location (file_id, line, column)
+  // - exports
+  // + used files
+
+  if (mod->isReference()) return; // skip external references
+
+  m_output.openHash()
+    .addFieldQuotedString("name", mod->name());
+
+  generatePerlUserDefinedSection(mod, mod->getMemberGroups());
+
+  if (!mod->getClasses().empty())
+  {
+    m_output.openList("classes");
+    for (const auto &cd : mod->getClasses())
+      m_output.openHash()
+	.addFieldQuotedString("name", cd->name())
+	.closeHash();
+    m_output.closeList();
+  }
+
+  if (!mod->getConcepts().empty())
+  {
+    m_output.openList("concepts");
+    for (const auto &cd : mod->getConcepts())
+      m_output.openHash()
+	.addFieldQuotedString("name", cd->name())
+	.closeHash();
+    m_output.closeList();
+  }
+
+  generatePerlModSection(mod,mod->getMemberList(MemberListType_decTypedefMembers),"typedefs");
+  generatePerlModSection(mod,mod->getMemberList(MemberListType_decEnumMembers),"enums");
+  generatePerlModSection(mod,mod->getMemberList(MemberListType_decFuncMembers),"functions");
+  generatePerlModSection(mod,mod->getMemberList(MemberListType_decVarMembers),"variables");
+
+  addPerlModDocBlock(m_output,"brief",mod->getDefFileName(),mod->getDefLine(),0,0,mod->briefDescription());
+  addPerlModDocBlock(m_output,"detailed",mod->getDefFileName(),mod->getDefLine(),0,0,mod->documentation());
+
+  if (!mod->getUsedFiles().empty())
+  {
+    m_output.openList("files");
+    for (const auto &fd : mod->getUsedFiles())
+      m_output.openHash()
+	.addFieldQuotedString("name", fd->name())
+	.closeHash();
+    m_output.closeList();
+  }
+
+  m_output.closeHash();
+}
+
 void PerlModGenerator::generatePerlModForNamespace(const NamespaceDef *nd)
 {
   // + contained class definitions
@@ -2011,6 +2059,26 @@ void PerlModGenerator::generatePerlModForGroup(const GroupDef *gd)
     m_output.closeList();
   }
 
+  if (!gd->getConcepts().empty())
+  {
+    m_output.openList("concepts");
+    for (const auto &cd : gd->getConcepts())
+      m_output.openHash()
+	.addFieldQuotedString("name", cd->name())
+	.closeHash();
+    m_output.closeList();
+  }
+
+  if (!gd->getModules().empty())
+  {
+    m_output.openList("modules");
+    for (const auto &mod : gd->getModules())
+      m_output.openHash()
+	.addFieldQuotedString("name", mod->name())
+	.closeHash();
+    m_output.closeList();
+  }
+
   if (!gd->getNamespaces().empty())
   {
     m_output.openList("namespaces");
@@ -2095,6 +2163,11 @@ bool PerlModGenerator::generatePerlModOutput()
     generatePerlModForConcept(cd.get());
   m_output.closeList();
 
+  m_output.openList("modules");
+  for (const auto &mod : ModuleManager::instance().modules())
+    generatePerlModForModule(mod.get());
+  m_output.closeList();
+
   m_output.openList("namespaces");
   for (const auto &nd : *Doxygen::namespaceLinkedMap)
     generatePerlModForNamespace(nd.get());
@@ -2129,6 +2202,7 @@ bool PerlModGenerator::generatePerlModOutput()
   m_output.closeList();
 
   m_output.closeHash().add(";\n1;\n");
+  m_output.reset();
   return true;
 }
 
