@@ -1,8 +1,6 @@
 /******************************************************************************
  *
- *
- *
- * Copyright (C) 1997-2015 by Parker Waechter & Dimitri van Heesch.
+ * Copyright (C) 1997-2023 by Parker Waechter & Dimitri van Heesch.
  *
  * Style sheet additions by Alexander Bartolich
  *
@@ -49,6 +47,7 @@
 #include "debug.h"
 #include "datetime.h"
 #include "outputlist.h"
+#include "moduledef.h"
 
 //#define DBG_RTF(x) x;
 #define DBG_RTF(x)
@@ -240,7 +239,7 @@ void RTFCodeGenerator::writeLineNumber(const QCString &ref,const QCString &fileN
   m_col=0;
 }
 
-void RTFCodeGenerator::startCodeLine(bool)
+void RTFCodeGenerator::startCodeLine(int)
 {
   m_doxyCodeLineOpen = true;
   m_col=0;
@@ -703,6 +702,10 @@ void RTFGenerator::startIndexSection(IndexSection is)
       //Introduction
       beginRTFChapter();
       break;
+    case IndexSection::isTopicIndex:
+      //Topic Index
+      beginRTFChapter();
+      break;
     case IndexSection::isModuleIndex:
       //Module Index
       beginRTFChapter();
@@ -736,9 +739,9 @@ void RTFGenerator::startIndexSection(IndexSection is)
       //Related Page Index
       beginRTFChapter();
       break;
-    case IndexSection::isModuleDocumentation:
+    case IndexSection::isTopicDocumentation:
       {
-        //Module Documentation
+        //Topic Documentation
         for (const auto &gd : *Doxygen::groupLinkedMap)
         {
           if (!gd->isReference())
@@ -749,6 +752,19 @@ void RTFGenerator::startIndexSection(IndexSection is)
         }
       }
       break;
+    case IndexSection::isModuleDocumentation:
+      {
+        //Module Documentation
+        for (const auto &mod : ModuleManager::instance().modules())
+        {
+          if (!mod->isReference() && mod->isPrimaryInterface())
+          {
+            beginRTFChapter();
+            break;
+          }
+        }
+      }
+      break;
     case IndexSection::isDirDocumentation:
       {
         //Directory Documentation
@@ -962,6 +978,11 @@ void RTFGenerator::endIndexSection(IndexSection is)
         writePageLink(Doxygen::mainPage->getOutputFileBase(), TRUE);
       }
       break;
+    case IndexSection::isTopicIndex:
+      m_t << "\\par " << rtf_Style_Reset << "\n";
+      m_t << "{\\tc \\v " << theTranslator->trTopicIndex() << "}\n";
+      m_t << "{\\field\\fldedit{\\*\\fldinst INCLUDETEXT \"topics.rtf\" \\\\*MERGEFORMAT}{\\fldrslt includedstuff}}\n";
+      break;
     case IndexSection::isModuleIndex:
       m_t << "\\par " << rtf_Style_Reset << "\n";
       m_t << "{\\tc \\v " << theTranslator->trModuleIndex() << "}\n";
@@ -1021,9 +1042,9 @@ void RTFGenerator::endIndexSection(IndexSection is)
       m_t << "{\\tc \\v " << theTranslator->trPageIndex() << "}\n";
       m_t << "{\\field\\fldedit{\\*\\fldinst INCLUDETEXT \"pages.rtf\" \\\\*MERGEFORMAT}{\\fldrslt includedstuff}}\n";
       break;
-    case IndexSection::isModuleDocumentation:
+    case IndexSection::isTopicDocumentation:
       {
-        m_t << "{\\tc \\v " << theTranslator->trModuleDocumentation() << "}\n";
+        m_t << "{\\tc \\v " << theTranslator->trTopicDocumentation() << "}\n";
         for (const auto &gd : *Doxygen::groupLinkedMap)
         {
           if (!gd->isReference() && !gd->isASubGroup())
@@ -1033,6 +1054,18 @@ void RTFGenerator::endIndexSection(IndexSection is)
         }
       }
       break;
+    case IndexSection::isModuleDocumentation:
+      {
+        m_t << "{\\tc \\v " << theTranslator->trModuleDocumentation() << "}\n";
+        for (const auto &mod : ModuleManager::instance().modules())
+        {
+          if (!mod->isReference() && mod->isPrimaryInterface())
+          {
+            writePageLink(mod->getOutputFileBase(), FALSE);
+          }
+        }
+      }
+      break;
     case IndexSection::isDirDocumentation:
       {
         bool first=true;
